/*  VQF Encoder 
 *  Copyright (C) 2000  Mel Gorman (mel@skynet.ie)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  This file deals with writing the header a VQF file
 */

#include <vqf.h>
#include <vqfencoder.h>


/* Gets text data from header */
static int getTextInfo(char *buf, char *ID, CHeaderManager *hManage)
{
  try
  {
    CStringChunk sChunk = hManage->GetPrimaryChunk(std::string(ID));
    strcpy(buf, sChunk.GetString().c_str());
  } catch (CHeaderManager::err_FailGetChunk)
  {
    sprintf(buf, "");
    return 0;
  }

  return 1;
}

/* DSiz is related to the length of the song in bytes */
void GetDSiz(CHeaderManager *hManage, int *DSiz)
{
  try {
    CDsizChunk dChunk(hManage->GetPrimaryChunk("DSIZ"));
    *DSiz = dChunk.GetInt();
  } catch (CHeaderManager::err_FailGetChunk)
  {
    TRACE("GetDSiz: Failed to get dsiz\n");
    return;
  }
}

int vqfReadHeader(BFILE *file, headerInfo *header)
{
  /* Read in header */
  TRACE("vqfReadHeader: Opening BS header\n");
  CChunkChunk *tChunk = TvqGetBsHeaderInfo(file);
  if (tChunk == NULL) return -1; 

  /* Parse out header */
  TRACE("vqfReadHeader: Getting header manager\n");
  CHeaderManager *hManage;
  hManage = CHeaderManager::Create(*tChunk); 
  if (hManage == NULL) return -1;
  delete tChunk;

  /* Get the title from the header */
  TRACE("vqfReadHeader: Getting Primary Chunk\n");
  sprintf(header->ID, "%s", hManage->GetID().c_str());
  CCommChunk commChunk(hManage->GetPrimaryChunk("COMM"));
  header->channelMode   = commChunk.GetChannelMode();
  header->bitRate       = commChunk.GetBitRate();
  header->samplingRate  = commChunk.GetSamplingRate();
  header->securityLevel = commChunk.GetSecurityLevel();

  getTextInfo(header->Name, "NAME", hManage);
  getTextInfo(header->Auth, "AUTH", hManage);
  getTextInfo(header->Comt, "COMT", hManage);
  getTextInfo(header->Cpyr, "(c) ", hManage);
  getTextInfo(header->File, "FILE", hManage);

  TRACE("vqfReadHeader: Getting dsiz\n");
  GetDSiz(hManage, &(header->Dsiz));

}

int vqfWriteHeader(char *filename, headerInfo *header)
{
  TRACE("vqfWriteHeader: Initializing\n");
  char *tempFilename = new char[strlen(filename) + 6];
  strcpy(tempFilename, filename);
  strcat(tempFilename, ".tmp");

  /* Status flag for Tvq functions */
  int status;

  /* Reading old header */
  TRACE("vqfWriteHeader: Reading old header\n");
  headerInfo oldHeader;
  BFILE *inFile = bopen(filename, "r");
  if (!inFile)
  {
    printf("Failed to open VQF file(%s)\n", filename);
    return -1;
  }
  if (vqfReadHeader(inFile, &oldHeader) == -1)
  {
    printf("Failed to read header\n");
    return -1;
  }

  /* Encoder specific info */
  TRACE("vqfWriteHeader: Setting encoder specific information\n");
  encSpecificInfo encInfo;
  memset(&encInfo, 0, sizeof(encSpecificInfo));
  encInfo.N_CAN_GLOBAL = 16;

  /* Init encoder */
  TRACE("vqfWriteHeader: Initiliazing encoder dll\n");
  INDEX index;
  if (TvqEncInitialize(header, &encInfo, &index, 0) != 0) { TRACE("vqfWriteHeader: Failed to init encoder\n"); return -1; }

  /* Open file */
  TRACE("vqfWriteHeader: Opening output file\n");
  BFILE *outFile = bopen(tempFilename, "w");
  if (!outFile) return -1;

  /* Init writer */
  TRACE("vqfWriteHeader: Initialising writer\n");
  TvqInitBsWriter();

  /* Chunk */
  TRACE("vqfWriteHeader: Creating chunk\n");
  CChunkChunk *chunk = TvqCreateHeaderChunk(header, "header_info");
  if (!chunk) return -1;

  /* Write the header */
  TRACE("vqfWriteHeader: Writing new header\n");
  TvqPutBsHeaderInfo(outFile, *chunk);
  delete chunk;

  TRACE("vqfWriteHeader: Transferring frames\n");
  if (TvqInitBsReader(&oldHeader) != 0) { TRACE("vqfWriteHeader: Failed to init reader\n"); return -1; }

  while (TvqReadBsFrame(&index, inFile) != 0)
    TvqWriteBsFrame(&index, outFile);

  TRACE("vqfWriteHeader: Closing files\n");
  bclose (inFile);
  bclose (outFile);
  
  TRACE("vqfWriteHeader: Swapping files\n");
  unlink(filename);
  rename(tempFilename, filename);

  TRACE("vqfWriteHeader: Returning successfully\n");
  return 1;
}
